@@ -4,8 +4,9 @@ extern crate gcc;
 use std::env;
 use std::ffi::OsString;
 use std::fs;
-use std::path::PathBuf;
+use std::path::{PathBuf, Path};
 use std::process::Command;
+use std::io::ErrorKind;
 
 macro_rules! t {
     ($e:expr) => (match $e {
@@ -14,7 +15,6 @@ macro_rules! t {
     })
 }
 
-#[allow(deprecated)] // needed for `connect()`, since Rust 1.1 is supported
 fn main() {
     let target = env::var("TARGET").unwrap();
     let host = env::var("HOST").unwrap();
@@ -28,11 +28,28 @@ fn main() {
         return println!("cargo:rustc-flags=-l curl");
     }
 
+    // Illumos/Solaris requires explicit linking with libnsl
+    if target.contains("solaris") {
+        println!("cargo:rustc-flags=-l nsl");
+    }
+
     // Next, fall back and try to use pkg-config if its available.
-    match pkg_config::find_library("libcurl") {
-        Ok(..) => return,
-        Err(e) => println!("Couldn't find libcurl from \
-                           pkgconfig ({:?}), compiling it from source...", e),
+    if !target.contains("windows") {
+        match pkg_config::find_library("libcurl") {
+            Ok(lib) => {
+                for path in lib.include_paths.iter() {
+                    println!("cargo:include={}", path.display());
+                }
+                return
+            }
+            Err(e) => println!("Couldn't find libcurl from \
+                               pkgconfig ({:?}), compiling it from source...", e),
+        }
+    }
+
+    if !Path::new("curl/.git").exists() {
+        let _ = Command::new("git").args(&["submodule", "update", "--init"])
+                                   .status();
     }
 
     println!("cargo:rustc-link-search={}/lib", dst.display());
@@ -40,7 +57,8 @@ fn main() {
     println!("cargo:root={}", dst.display());
     println!("cargo:include={}/include", dst.display());
     if windows {
-        println!("cargo:rustc-flags=-l ws2_32");
+        println!("cargo:rustc-link-lib=ws2_32");
+        println!("cargo:rustc-link-lib=crypt32");
     }
 
     // MSVC builds are just totally different
@@ -48,6 +66,9 @@ fn main() {
         return build_msvc(&target);
     }
 
+    let openssl_root = register_dep("OPENSSL");
+    let zlib_root = register_dep("Z");
+
     let cfg = gcc::Config::new();
     let compiler = cfg.get_compiler();
 
@@ -59,35 +80,68 @@ fn main() {
         cflags.push(arg);
         cflags.push(" ");
     }
+
+    // Can't run ./configure directly on msys2 b/c we're handing in
+    // Windows-style paths (those starting with C:\), but it chokes on those.
+    // For that reason we build up a shell script with paths converted to
+    // posix versions hopefully...
+    //
+    // Also apparently the buildbots choke unless we manually set LD, who knows
+    // why?!
     cmd.env("CC", compiler.path())
        .env("CFLAGS", cflags)
        .env("LD", &which("ld").unwrap())
+       .env("VERBOSE", "1")
        .current_dir(&dst.join("build"))
-       .arg(src.join("curl/configure").to_str().unwrap()
-               .replace("C:\\", "/c/")
-               .replace("\\", "/"));
+       .arg(msys_compatible(&src.join("curl/configure")));
+
+    // For now this build script doesn't support paths with spaces in them. This
+    // is arguably a but in curl's configure script, but we could also try to
+    // paper over it by using a tmp directory which *doesn't* have spaces in it.
+    // As of now though that's not implemented so just give a nicer error for
+    // the time being.
+    let wants_space_error = windows &&
+        (dst.to_str().map(|s| s.contains(" ")).unwrap_or(false) ||
+         src.to_str().map(|s| s.contains(" ")).unwrap_or(false));
+    if wants_space_error {
+        panic!("\n\nunfortunately ./configure of libcurl is known to \
+                fail if there's a space in the path to the current \
+                directory\n\n\
+                there's a space in either\n  {}\n  {}\nand this will cause the \
+                build to fail\n\n\
+                the MSVC build should work with a directory that has \
+                spaces in it, and it would also work to move this to a \
+                different directory without spaces\n\n",
+               src.display(), dst.display())
+    }
+
     if windows {
         cmd.arg("--with-winssl");
     } else {
         cmd.arg("--without-ca-bundle");
         cmd.arg("--without-ca-path");
-
-        // DEP_OPENSSL_ROOT was what cargo used to use via overrides, and
-        // OPENSSL_ROOT_DIR is also used by libssh2 which cargo uses and can be
-        // set by various build scripts.
-        let openssl_root = env::var("DEP_OPENSSL_ROOT").or_else(|_| {
-            env::var("OPENSSL_ROOT_DIR")
-        });
-        if let Ok(s) = openssl_root {
-            cmd.arg(format!("--with-ssl={}", s));
-        }
+    }
+    if let Some(root) = openssl_root {
+        cmd.arg(format!("--with-ssl={}", root.display()));
+    }
+    if let Some(root) = zlib_root {
+        cmd.arg(format!("--with-zlib={}", root.display()));
     }
     cmd.arg("--enable-static=yes");
     cmd.arg("--enable-shared=no");
-    cmd.arg("--enable-optimize");
-    cmd.arg(format!("--prefix={}", dst.display()));
+    match &env::var("PROFILE").unwrap()[..] {
+        "bench" | "release" => {
+            cmd.arg("--enable-optimize");
+        }
+        _ => {
+            cmd.arg("--enable-debug");
+            cmd.arg("--disable-optimize");
+        }
+    }
+    cmd.arg(format!("--prefix={}", msys_compatible(&dst)));
 
-    if target != host {
+    if target != host &&
+       (!target.contains("windows") || !host.contains("windows")) {
         cmd.arg(format!("--host={}", host));
         cmd.arg(format!("--target={}", target));
     }
@@ -108,38 +162,35 @@ fn main() {
     cmd.arg("--disable-smtp");
     cmd.arg("--disable-gopher");
     cmd.arg("--disable-manual");
+    cmd.arg("--disable-smb");
+    cmd.arg("--disable-sspi");
 
-    // Can't run ./configure directly on msys2 b/c we're handing in
-    // Windows-style paths (those starting with C:\), but it chokes on those.
-    // For that reason we build up a shell script with paths converted to
-    // posix versions hopefully...
-    //
-    // Also apparently the buildbots choke unless we manually set LD, who knows
-    // why?!
-    run(&mut cmd);
+    run(&mut cmd, "sh");
     run(Command::new(make())
                 .arg(&format!("-j{}", env::var("NUM_JOBS").unwrap()))
-                .current_dir(&dst.join("build")));
-
-    // Don't run `make install` because apparently it's a little buggy on mingw
-    // for windows.
-    let _ = fs::create_dir_all(&dst.join("lib/pkgconfig"));
-
-    // Which one does windows generate? Who knows!
-    let p1 = dst.join("build/lib/.libs/libcurl.a");
-    let p2 = dst.join("build/lib/.libs/libcurl.lib");
-    if fs::metadata(&p1).is_ok() {
-        t!(fs::copy(&p1, &dst.join("lib/libcurl.a")));
-    } else {
-        t!(fs::copy(&p2, &dst.join("lib/libcurl.a")));
-    }
-    t!(fs::copy(&dst.join("build/libcurl.pc"),
-                  &dst.join("lib/pkgconfig/libcurl.pc")));
+                .current_dir(&dst.join("build")), "make");
+    run(Command::new(make())
+                .arg("install")
+                .current_dir(&dst.join("build")), "make");
 }
 
-fn run(cmd: &mut Command) {
+fn run(cmd: &mut Command, program: &str) {
     println!("running: {:?}", cmd);
-    assert!(t!(cmd.status()).success());
+    let status = match cmd.status() {
+        Ok(status) => status,
+        Err(ref e) if e.kind() == ErrorKind::NotFound => {
+            fail(&format!("failed to execute command: {}\nis `{}` not installed?",
+                          e, program));
+        }
+        Err(e) => fail(&format!("failed to execute command: {}", e)),
+    };
+    if !status.success() {
+        fail(&format!("command did not execute successfully, got: {}", status));
+    }
+}
+
+fn fail(s: &str) -> ! {
+    panic!("\n{}\n\nbuild script failed, must exit now", s)
 }
 
 fn make() -> &'static str {
@@ -154,6 +205,40 @@ fn which(cmd: &str) -> Option<PathBuf> {
     })
 }
 
+fn msys_compatible(path: &Path) -> String {
+    let path = path.to_str().unwrap();
+    if !cfg!(windows) {
+        return path.to_string()
+    }
+    path.replace("C:\\", "/c/")
+        .replace("\\", "/")
+}
+
+fn register_dep(dep: &str) -> Option<PathBuf> {
+    if let Some(s) = env::var_os(&format!("DEP_{}_ROOT", dep)) {
+        prepend("PKG_CONFIG_PATH", Path::new(&s).join("lib/pkgconfig"));
+        return Some(s.into())
+    }
+    if let Some(s) = env::var_os(&format!("DEP_{}_INCLUDE", dep)) {
+        let root = Path::new(&s).parent().unwrap();
+        env::set_var(&format!("DEP_{}_ROOT", dep), root);
+        let path = root.join("lib/pkgconfig");
+        if path.exists() {
+            prepend("PKG_CONFIG_PATH", path);
+            return Some(root.to_path_buf())
+        }
+    }
+
+    return None;
+
+    fn prepend(var: &str, val: PathBuf) {
+        let prefix = env::var(var).unwrap_or(String::new());
+        let mut v = vec![val];
+        v.extend(env::split_paths(&prefix));
+        env::set_var(var, &env::join_paths(v).unwrap());
+    }
+}
+
 fn build_msvc(target: &str) {
     let cmd = gcc::windows_registry::find(target, "nmake.exe");
     let mut cmd = cmd.unwrap_or(Command::new("nmake.exe"));
@@ -192,7 +277,7 @@ fn build_msvc(target: &str) {
         let _ = fs::remove_file(&inc.join("lib/zlib_a.lib"));
         t!(fs::hard_link(inc.join("lib/zlib.lib"), inc.join("lib/zlib_a.lib")));
     }
-    run(&mut cmd);
+    run(&mut cmd, "nmake");
 
     let name = format!("libcurl-vc-{}-release-static-zlib-static-\
                         ipv6-sspi-winssl", machine);
