@@ -2,15 +2,18 @@
 
 use std::io::prelude::*;
 use std::io;
+use std::mem;
 
-use raw;
+use bufreader::BufReader;
+use zio;
+use {Compress, Decompress};
 
 /// A ZLIB encoder, or compressor.
 ///
 /// This structure implements a `Write` interface and takes a stream of
 /// uncompressed data, writing the compressed data to the wrapped writer.
 pub struct EncoderWriter<W: Write> {
-    inner: raw::EncoderWriter<W>,
+    inner: zio::Writer<W, Compress>,
 }
 
 /// A ZLIB encoder, or compressor.
@@ -18,7 +21,16 @@ pub struct EncoderWriter<W: Write> {
 /// This structure implements a `Read` interface and will read uncompressed
 /// data from an underlying stream and emit a stream of compressed data.
 pub struct EncoderReader<R: Read> {
-    inner: raw::EncoderReader<R>,
+    inner: EncoderReaderBuf<BufReader<R>>,
+}
+
+/// A ZLIB encoder, or compressor.
+///
+/// This structure implements a `BufRead` interface and will read uncompressed
+/// data from an underlying stream and emit a stream of compressed data.
+pub struct EncoderReaderBuf<R: BufRead> {
+    obj: R,
+    data: Compress,
 }
 
 /// A ZLIB decoder, or decompressor.
@@ -26,7 +38,16 @@ pub struct EncoderReader<R: Read> {
 /// This structure implements a `Read` interface and takes a stream of
 /// compressed data as input, providing the decompressed data when read from.
 pub struct DecoderReader<R: Read> {
-    inner: raw::DecoderReader<R>,
+    inner: DecoderReaderBuf<BufReader<R>>,
+}
+
+/// A ZLIB decoder, or decompressor.
+///
+/// This structure implements a `BufRead` interface and takes a stream of
+/// compressed data as input, providing the decompressed data when read from.
+pub struct DecoderReaderBuf<R: BufRead> {
+    obj: R,
+    data: Decompress,
 }
 
 /// A ZLIB decoder, or decompressor.
@@ -34,7 +55,7 @@ pub struct DecoderReader<R: Read> {
 /// This structure implements a `Write` and will emit a stream of decompressed
 /// data when fed a stream of compressed data.
 pub struct DecoderWriter<W: Write> {
-    inner: raw::DecoderWriter<W>,
+    inner: zio::Writer<W, Decompress>,
 }
 
 impl<W: Write> EncoderWriter<W> {
@@ -45,10 +66,7 @@ impl<W: Write> EncoderWriter<W> {
     /// be flushed.
     pub fn new(w: W, level: ::Compression) -> EncoderWriter<W> {
         EncoderWriter {
-            inner: raw::EncoderWriter::new(w,
-                                           level,
-                                           false,
-                                           Vec::with_capacity(32 * 1024)),
+            inner: zio::Writer::new(w, Compress::new(level, true)),
         }
     }
 
@@ -65,7 +83,8 @@ impl<W: Write> EncoderWriter<W> {
     /// this encoder will be the compressed into the stream `w` provided.
     pub fn reset(&mut self, w: W) -> io::Result<W> {
         try!(self.inner.finish());
-        Ok(self.inner.reset(w))
+        self.inner.data.reset();
+        Ok(self.inner.replace(w))
     }
 
     /// Consumes this encoder, flushing the output stream.
@@ -93,7 +112,7 @@ impl<R: Read> EncoderReader<R> {
     /// stream and emit the compressed stream.
     pub fn new(r: R, level: ::Compression) -> EncoderReader<R> {
         EncoderReader {
-            inner: raw::EncoderReader::new(r, level, false, vec![0; 32 * 1024]),
+            inner: EncoderReaderBuf::new(BufReader::new(r), level),
         }
     }
 
@@ -105,12 +124,26 @@ impl<R: Read> EncoderReader<R> {
     /// stream. Future data read from this encoder will be the compressed
     /// version of `r`'s data.
     pub fn reset(&mut self, r: R) -> R {
-        self.inner.reset(r)
+        self.inner.data.reset();
+        self.inner.obj.reset(r)
+    }
+
+    /// Acquires a reference to the underlying stream
+    pub fn get_ref(&self) -> &R {
+        self.inner.get_ref().get_ref()
+    }
+
+    /// Acquires a mutable reference to the underlying stream
+    ///
+    /// Note that mutation of the stream may result in surprising results if
+    /// this encoder is continued to be used.
+    pub fn get_mut(&mut self) -> &mut R {
+        self.inner.get_mut().get_mut()
     }
 
     /// Consumes this encoder, returning the underlying reader.
     pub fn into_inner(self) -> R {
-        self.inner.into_inner()
+        self.inner.into_inner().into_inner()
     }
 }
 
@@ -120,6 +153,53 @@ impl<R: Read> Read for EncoderReader<R> {
     }
 }
 
+impl<R: BufRead> EncoderReaderBuf<R> {
+    /// Creates a new encoder which will read uncompressed data from the given
+    /// stream and emit the compressed stream.
+    pub fn new(r: R, level: ::Compression) -> EncoderReaderBuf<R> {
+        EncoderReaderBuf {
+            obj: r,
+            data: Compress::new(level, true),
+        }
+    }
+
+    /// Resets the state of this encoder entirely, swapping out the input
+    /// stream for another.
+    ///
+    /// This function will reset the internal state of this encoder and replace
+    /// the input stream with the one provided, returning the previous input
+    /// stream. Future data read from this encoder will be the compressed
+    /// version of `r`'s data.
+    pub fn reset(&mut self, r: R) -> R {
+        self.data.reset();
+        mem::replace(&mut self.obj, r)
+    }
+
+    /// Acquires a reference to the underlying stream
+    pub fn get_ref(&self) -> &R {
+        &self.obj
+    }
+
+    /// Acquires a mutable reference to the underlying stream
+    ///
+    /// Note that mutation of the stream may result in surprising results if
+    /// this encoder is continued to be used.
+    pub fn get_mut(&mut self) -> &mut R {
+        &mut self.obj
+    }
+
+    /// Consumes this encoder, returning the underlying reader.
+    pub fn into_inner(self) -> R {
+        self.obj
+    }
+}
+
+impl<R: BufRead> Read for EncoderReaderBuf<R> {
+    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
+        zio::read(&mut self.obj, &mut self.data, buf)
+    }
+}
+
 impl<R: Read> DecoderReader<R> {
     /// Creates a new decoder which will decompress data read from the given
     /// stream.
@@ -132,7 +212,9 @@ impl<R: Read> DecoderReader<R> {
     /// Note that the specified buffer will only be used up to its current
     /// length. The buffer's capacity will also not grow over time.
     pub fn new_with_buf(r: R, buf: Vec<u8>) -> DecoderReader<R> {
-        DecoderReader { inner: raw::DecoderReader::new(r, false, buf) }
+        DecoderReader {
+            inner: DecoderReaderBuf::new(BufReader::with_buf(buf, r)),
+        }
     }
 
     /// Resets the state of this decoder entirely, swapping out the input
@@ -143,12 +225,26 @@ impl<R: Read> DecoderReader<R> {
     /// stream. Future data read from this decoder will be the decompressed
     /// version of `r`'s data.
     pub fn reset(&mut self, r: R) -> R {
-        self.inner.reset(r, false)
+        self.inner.data = Decompress::new(true);
+        self.inner.obj.reset(r)
+    }
+
+    /// Acquires a reference to the underlying stream
+    pub fn get_ref(&self) -> &R {
+        self.inner.get_ref().get_ref()
+    }
+
+    /// Acquires a mutable reference to the underlying stream
+    ///
+    /// Note that mutation of the stream may result in surprising results if
+    /// this encoder is continued to be used.
+    pub fn get_mut(&mut self) -> &mut R {
+        self.inner.get_mut().get_mut()
     }
 
     /// Consumes this decoder, returning the underlying reader.
     pub fn into_inner(self) -> R {
-        self.inner.into_inner()
+        self.inner.into_inner().into_inner()
     }
 
     /// Returns the number of bytes that the decompressor has consumed.
@@ -171,6 +267,66 @@ impl<R: Read> Read for DecoderReader<R> {
     }
 }
 
+impl<R: BufRead> DecoderReaderBuf<R> {
+    /// Creates a new decoder which will decompress data read from the given
+    /// stream.
+    pub fn new(r: R) -> DecoderReaderBuf<R> {
+        DecoderReaderBuf {
+            obj: r,
+            data: Decompress::new(true),
+        }
+    }
+
+    /// Resets the state of this decoder entirely, swapping out the input
+    /// stream for another.
+    ///
+    /// This will reset the internal state of this decoder and replace the
+    /// input stream with the one provided, returning the previous input
+    /// stream. Future data read from this decoder will be the decompressed
+    /// version of `r`'s data.
+    pub fn reset(&mut self, r: R) -> R {
+        self.data = Decompress::new(true);
+        mem::replace(&mut self.obj, r)
+    }
+
+    /// Acquires a reference to the underlying stream
+    pub fn get_ref(&self) -> &R {
+        &self.obj
+    }
+
+    /// Acquires a mutable reference to the underlying stream
+    ///
+    /// Note that mutation of the stream may result in surprising results if
+    /// this encoder is continued to be used.
+    pub fn get_mut(&mut self) -> &mut R {
+        &mut self.obj
+    }
+
+    /// Consumes this decoder, returning the underlying reader.
+    pub fn into_inner(self) -> R {
+        self.obj
+    }
+
+    /// Returns the number of bytes that the decompressor has consumed.
+    ///
+    /// Note that this will likely be smaller than what the decompressor
+    /// actually read from the underlying stream due to buffering.
+    pub fn total_in(&self) -> u64 {
+        self.data.total_in()
+    }
+
+    /// Returns the number of bytes that the decompressor has produced.
+    pub fn total_out(&self) -> u64 {
+        self.data.total_out()
+    }
+}
+
+impl<R: BufRead> Read for DecoderReaderBuf<R> {
+    fn read(&mut self, into: &mut [u8]) -> io::Result<usize> {
+        zio::read(&mut self.obj, &mut self.data, into)
+    }
+}
+
 impl<W: Write> DecoderWriter<W> {
     /// Creates a new decoder which will write uncompressed data to the stream.
     ///
@@ -178,9 +334,7 @@ impl<W: Write> DecoderWriter<W> {
     /// be flushed.
     pub fn new(w: W) -> DecoderWriter<W> {
         DecoderWriter {
-            inner: raw::DecoderWriter::new(w,
-                                           false,
-                                           Vec::with_capacity(32 * 1024)),
+            inner: zio::Writer::new(w, Decompress::new(true)),
         }
     }
 
@@ -193,7 +347,8 @@ impl<W: Write> DecoderWriter<W> {
     /// the output stream `w`.
     pub fn reset(&mut self, w: W) -> io::Result<W> {
         try!(self.inner.finish());
-        Ok(self.inner.reset(w, false))
+        self.inner.data = Decompress::new(true);
+        Ok(self.inner.replace(w))
     }
 
     /// Consumes this encoder, flushing the output stream.
@@ -211,13 +366,13 @@ impl<W: Write> DecoderWriter<W> {
     /// Note that this will likely be smaller than the number of bytes
     /// successfully written to this stream due to internal buffering.
     pub fn total_in(&self) -> u64 {
-        self.inner.total_in()
+        self.inner.data.total_in()
     }
 
     /// Returns the number of bytes that the decompressor has written to its
     /// output stream.
     pub fn total_out(&self) -> u64 {
-        self.inner.total_out()
+        self.inner.data.total_out()
     }
 }
 
@@ -257,6 +412,16 @@ mod tests {
         assert!(ret == real);
     }
 
+    #[test]
+    fn drop_writes() {
+        let mut data = Vec::new();
+        EncoderWriter::new(&mut data, Default).write_all(b"foo").unwrap();
+        let mut r = DecoderReader::new(&data[..]);
+        let mut ret = Vec::new();
+        r.read_to_end(&mut ret).unwrap();
+        assert!(ret == b"foo");
+    }
+
     #[test]
     fn total_in() {
         let mut real = Vec::new();
