@@ -46,12 +46,16 @@
 #![cfg_attr(test, deny(warnings))]
 #![deny(missing_docs)]
 
+#[cfg(feature = "parallel")]
+extern crate rayon;
+
 use std::env;
 use std::ffi::{OsString, OsStr};
 use std::fs;
 use std::io;
 use std::path::{PathBuf, Path};
 use std::process::{Command, Stdio};
+use std::io::{BufReader, BufRead, Write};
 
 #[cfg(windows)]
 mod registry;
@@ -70,7 +74,7 @@ pub struct Config {
     target: Option<String>,
     host: Option<String>,
     out_dir: Option<PathBuf>,
-    opt_level: Option<u32>,
+    opt_level: Option<String>,
     debug: Option<bool>,
     env: Vec<(OsString, OsString)>,
     compiler: Option<PathBuf>,
@@ -246,7 +250,16 @@ impl Config {
     /// This option is automatically scraped from the `OPT_LEVEL` environment
     /// variable by build scripts, so it's not required to call this function.
     pub fn opt_level(&mut self, opt_level: u32) -> &mut Config {
-        self.opt_level = Some(opt_level);
+        self.opt_level = Some(opt_level.to_string());
+        self
+    }
+
+    /// Configures the optimization level of the generated object files.
+    ///
+    /// This option is automatically scraped from the `OPT_LEVEL` environment
+    /// variable by build scripts, so it's not required to call this function.
+    pub fn opt_level_str(&mut self, opt_level: &str) -> &mut Config {
+        self.opt_level = Some(opt_level.to_string());
         self
     }
 
@@ -325,12 +338,19 @@ impl Config {
         let dst = self.get_out_dir();
 
         let mut objects = Vec::new();
+        let mut src_dst = Vec::new();
         for file in self.files.iter() {
             let obj = dst.join(file).with_extension("o");
-            self.compile_object(file, &obj);
+            let obj = if !obj.starts_with(&dst) {
+                dst.join(obj.file_name().unwrap())
+            } else {
+                obj
+            };
+            fs::create_dir_all(&obj.parent().unwrap()).unwrap();
+            src_dst.push((file.to_path_buf(), obj.clone()));
             objects.push(obj);
         }
-
+        self.compile_objects(&src_dst);
         self.assemble(lib_name, &dst.join(output), &objects);
 
         self.print(&format!("cargo:rustc-link-lib=static={}",
@@ -345,6 +365,30 @@ impl Config {
         }
     }
 
+    #[cfg(feature = "parallel")]
+    fn compile_objects(&self, objs: &[(PathBuf, PathBuf)]) {
+        use self::rayon::prelude::*;
+
+        let mut cfg = rayon::Configuration::new();
+        if let Ok(amt) = env::var("NUM_JOBS") {
+            if let Ok(amt) = amt.parse() {
+                cfg = cfg.set_num_threads(amt);
+            }
+        }
+        drop(rayon::initialize(cfg));
+
+        objs.par_iter().weight_max().for_each(|&(ref src, ref dst)| {
+            self.compile_object(src, dst)
+        })
+    }
+
+    #[cfg(not(feature = "parallel"))]
+    fn compile_objects(&self, objs: &[(PathBuf, PathBuf)]) {
+        for &(ref src, ref dst) in objs {
+            self.compile_object(src, dst);
+        }
+    }
+
     fn compile_object(&self, file: &Path, dst: &Path) {
         let is_asm = file.extension().and_then(|s| s.to_str()) == Some("asm");
         let msvc = self.get_target().contains("msvc");
@@ -359,7 +403,6 @@ impl Config {
             (cmd, compiler.path.file_name().unwrap()
                           .to_string_lossy().into_owned())
         };
-        fs::create_dir_all(&dst.parent().unwrap()).unwrap();
         if msvc && is_asm {
             cmd.arg("/Fo").arg(dst);
         } else if msvc {
@@ -404,11 +447,17 @@ impl Config {
         if msvc {
             cmd.args.push("/nologo".into());
             cmd.args.push("/MD".into()); // link against msvcrt.dll for now
-            if opt_level != 0 {
-                cmd.args.push("/O2".into());
+            match &opt_level[..] {
+                "z" | "s" => cmd.args.push("/Os".into()),
+                "2" => cmd.args.push("/O2".into()),
+                "1" => cmd.args.push("/O1".into()),
+                _ => {}
             }
             if target.contains("i686") {
                 cmd.args.push("/SAFESEH".into());
+            } else if target.contains("i586") {
+                cmd.args.push("/SAFESEH".into());
+                cmd.args.push("/ARCH:IA32".into());
             }
         } else if nvcc {
             cmd.args.push(format!("-O{}", opt_level).into());
@@ -428,7 +477,7 @@ impl Config {
         if target.contains("-ios") {
             self.ios_flags(&mut cmd);
         } else if !msvc {
-            if target.contains("i686") {
+            if target.contains("i686") || target.contains("i586") {
                 cmd.args.push("-m32".into());
             } else if target.contains("x86_64") || target.contains("powerpc64") {
                 cmd.args.push("-m64".into());
@@ -444,16 +493,38 @@ impl Config {
                 cmd.args.push("-static".into());
             }
 
-            if target == "armv7-unknown-linux-gnueabihf" {
+            if target.starts_with("armv7-unknown-linux-") {
                 cmd.args.push("-march=armv7-a".into());
             }
-            if target == "arm-unknown-linux-gnueabihf" {
+            if target.starts_with("armv7-linux-androideabi") {
+                cmd.args.push("-march=armv7-a".into());
+                cmd.args.push("-mfpu=vfpv3-d16".into());
+            }
+            if target.starts_with("arm-unknown-linux-") {
                 cmd.args.push("-march=armv6".into());
                 cmd.args.push("-marm".into());
             }
-            if target == "arm-unknown-linux-gnueabi" {
-                cmd.args.push("-march=armv6".into());
-                cmd.args.push("-marm".into());
+            if target.starts_with("i586-unknown-linux-") {
+                cmd.args.push("-march=pentium".into());
+            }
+            if target.starts_with("i686-unknown-linux-") {
+                cmd.args.push("-march=i686".into());
+            }
+            if target.starts_with("thumb") {
+                cmd.args.push("-mthumb".into());
+
+                if target.ends_with("eabihf") {
+                    cmd.args.push("-mfloat-abi=hard".into())
+                }
+            }
+            if target.starts_with("thumbv6m") {
+                cmd.args.push("-march=armv6-m".into());
+            }
+            if target.starts_with("thumbv7em") {
+                cmd.args.push("-march=armv7e-m".into());
+            }
+            if target.starts_with("thumbv7m") {
+                cmd.args.push("-march=armv7-m".into());
             }
         }
 
@@ -500,7 +571,7 @@ impl Config {
             }
         }
 
-        if target.contains("i686") {
+        if target.contains("i686") || target.contains("i586") {
             cmd.arg("/safeseh");
         }
         for flag in self.flags.iter() {
@@ -511,6 +582,10 @@ impl Config {
     }
 
     fn assemble(&self, lib_name: &str, dst: &Path, objects: &[PathBuf]) {
+        // Delete the destination if it exists as the `ar` tool at least on Unix
+        // appends to it, which we don't want.
+        let _ = fs::remove_file(&dst);
+
         let target = self.get_target();
         if target.contains("msvc") {
             let mut cmd = match self.archiver {
@@ -536,7 +611,7 @@ impl Config {
         } else {
             let ar = self.get_ar();
             let cmd = ar.file_name().unwrap().to_string_lossy();
-            run(self.cmd(&ar).arg("crus")
+            run(self.cmd(&ar).arg("crs")
                                  .arg(dst)
                                  .args(objects)
                                  .args(&self.objects), &cmd);
@@ -609,8 +684,22 @@ impl Config {
         } else {
             ("CC", "cl.exe", "gcc", "cc")
         };
-        self.get_var(env).ok().map(|env| {
-            Tool::new(PathBuf::from(env))
+        self.env_tool(env).map(|(tool, args)| {
+            let mut t = Tool::new(PathBuf::from(tool));
+            for arg in args {
+                t.args.push(arg.into());
+            }
+            return t
+        }).or_else(|| {
+            if target.contains("emscripten") {
+                if self.cpp {
+                    Some(Tool::new(PathBuf::from("em++")))
+                } else {
+                    Some(Tool::new(PathBuf::from("emcc")))
+                }
+            } else {
+                None
+            }
         }).or_else(|| {
             windows_registry::find_tool(&target, "cl.exe")
         }).unwrap_or_else(|| {
@@ -624,22 +713,42 @@ impl Config {
             } else if target.contains("android") {
                 format!("{}-{}", target, gnu)
             } else if self.get_host() != target {
-                let prefix = match &target[..] {
+                // CROSS_COMPILE is of the form: "arm-linux-gnueabi-"
+                let cc_env = self.getenv("CROSS_COMPILE");
+                let cross_compile = cc_env.as_ref().map(|s| s.trim_right_matches('-'));
+                let prefix = cross_compile.or(match &target[..] {
                     "aarch64-unknown-linux-gnu" => Some("aarch64-linux-gnu"),
                     "arm-unknown-linux-gnueabi" => Some("arm-linux-gnueabi"),
                     "arm-unknown-linux-gnueabihf"  => Some("arm-linux-gnueabihf"),
+                    "arm-unknown-linux-musleabi" => Some("arm-linux-musleabi"),
+                    "arm-unknown-linux-musleabihf"  => Some("arm-linux-musleabihf"),
+                    "arm-unknown-netbsdelf-eabi" => Some("arm--netbsdelf-eabi"),
+                    "armv6-unknown-netbsdelf-eabihf" => Some("armv6--netbsdelf-eabihf"),
                     "armv7-unknown-linux-gnueabihf" => Some("arm-linux-gnueabihf"),
-                    "powerpc-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
-                    "powerpc64-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
-                    "powerpc64le-unknown-linux-gnu" => Some("powerpc64le-linux-gnu"),
+                    "armv7-unknown-linux-musleabihf" => Some("arm-linux-musleabihf"),
+                    "armv7-unknown-netbsdelf-eabihf" => Some("armv7--netbsdelf-eabihf"),
+                    "i686-pc-windows-gnu" => Some("i686-w64-mingw32"),
+                    "i686-unknown-linux-musl" => Some("musl"),
+                    "i686-unknown-netbsdelf" => Some("i486--netbsdelf"),
                     "mips-unknown-linux-gnu" => Some("mips-linux-gnu"),
                     "mipsel-unknown-linux-gnu" => Some("mipsel-linux-gnu"),
-                    "i686-pc-windows-gnu" => Some("i686-w64-mingw32"),
+                    "mips64-unknown-linux-gnuabi64" => Some("mips64-linux-gnuabi64"),
+                    "mips64el-unknown-linux-gnuabi64" => Some("mips64el-linux-gnuabi64"),
+                    "powerpc-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
+                    "powerpc-unknown-netbsd" => Some("powerpc--netbsd"),
+                    "powerpc64-unknown-linux-gnu" => Some("powerpc-linux-gnu"),
+                    "powerpc64le-unknown-linux-gnu" => Some("powerpc64le-linux-gnu"),
+                    "s390x-unknown-linux-gnu" => Some("s390x-linux-gnu"),
+                    "thumbv6m-none-eabi" => Some("arm-none-eabi"),
+                    "thumbv7em-none-eabi" => Some("arm-none-eabi"),
+                    "thumbv7em-none-eabihf" => Some("arm-none-eabi"),
+                    "thumbv7m-none-eabi" => Some("arm-none-eabi"),
                     "x86_64-pc-windows-gnu" => Some("x86_64-w64-mingw32"),
-                    "x86_64-unknown-linux-musl" => Some("musl"),
                     "x86_64-rumprun-netbsd" => Some("x86_64-rumprun-netbsd"),
+                    "x86_64-unknown-linux-musl" => Some("musl"),
+                    "x86_64-unknown-netbsd" => Some("x86_64--netbsd"),
                     _ => None,
-                };
+                });
                 match prefix {
                     Some(prefix) => format!("{}-{}", prefix, gnu),
                     None => default.to_string(),
@@ -663,7 +772,7 @@ impl Config {
 
         match res {
             Some(res) => Ok(res),
-            None => Err("Could not get environment variable".to_string()),
+            None => Err("could not get environment variable".to_string()),
         }
     }
 
@@ -674,6 +783,19 @@ impl Config {
             .collect()
     }
 
+    fn env_tool(&self, name: &str) -> Option<(String, Vec<String>)> {
+        self.get_var(name).ok().map(|tool| {
+            let whitelist = ["ccache", "distcc"];
+            for t in whitelist.iter() {
+                if tool.starts_with(t) && tool[t.len()..].starts_with(" ") {
+                    return (t.to_string(),
+                            vec![tool[t.len()..].trim_left().to_string()])
+                }
+            }
+            (tool, Vec::new())
+        })
+    }
+
     /// Returns the default C++ standard library for the current target: `libc++`
     /// for OS X and `libstdc++` for anything else.
     fn get_cpp_link_stdlib(&self) -> Option<String> {
@@ -695,6 +817,8 @@ impl Config {
         }).unwrap_or_else(|| {
             if self.get_target().contains("android") {
                 PathBuf::from(format!("{}-ar", self.get_target()))
+            } else if self.get_target().contains("emscripten") {
+                PathBuf::from("emar")
             } else {
                 PathBuf::from("ar")
             }
@@ -709,9 +833,9 @@ impl Config {
         self.host.clone().unwrap_or_else(|| self.getenv_unwrap("HOST"))
     }
 
-    fn get_opt_level(&self) -> u32 {
-        self.opt_level.unwrap_or_else(|| {
-            self.getenv_unwrap("OPT_LEVEL").parse().unwrap()
+    fn get_opt_level(&self) -> String {
+        self.opt_level.as_ref().cloned().unwrap_or_else(|| {
+            self.getenv_unwrap("OPT_LEVEL")
         })
     }
 
@@ -793,7 +917,23 @@ impl Tool {
 
 fn run(cmd: &mut Command, program: &str) {
     println!("running: {:?}", cmd);
-    let status = match cmd.status() {
+    // Capture the standard error coming from these programs, and write it out
+    // with cargo:warning= prefixes. Note that this is a bit wonky to avoid
+    // requiring the output to be UTF-8, we instead just ship bytes from one
+    // location to another.
+    let spawn_result = match cmd.stderr(Stdio::piped()).spawn() {
+        Ok(mut child) => {
+            let stderr = BufReader::new(child.stderr.take().unwrap());
+            for line in stderr.split(b'\n').filter_map(|l| l.ok()) {
+                print!("cargo:warning=");
+                std::io::stdout().write_all(&line).unwrap();
+                println!("");
+            }
+            child.wait()
+        }
+        Err(e) => Err(e),
+    };
+    let status = match spawn_result {
         Ok(status) => status,
         Err(ref e) if e.kind() == io::ErrorKind::NotFound => {
             let extra = if cfg!(windows) {
