@@ -18,6 +18,16 @@ macro_rules! try_call {
     })
 }
 
+macro_rules! try_call_iter {
+    ($($f:tt)*) => {
+        match call!($($f)*) {
+            0 => {}
+            raw::GIT_ITEROVER => return None,
+            e => return Some(Err(::call::last_error(e)))
+        }
+    }
+}
+
 #[doc(hidden)]
 pub trait Convert<T> {
     fn convert(&self) -> T;
@@ -32,7 +42,7 @@ pub fn try(ret: libc::c_int) -> Result<libc::c_int, Error> {
     }
 }
 
-fn last_error(code: libc::c_int) -> Error {
+pub fn last_error(code: libc::c_int) -> Error {
     // Apparently libgit2 isn't necessarily guaranteed to set the last error
     // whenever a function returns a negative value!
     Error::last_error(code).unwrap_or_else(|| {
@@ -136,6 +146,7 @@ mod impls {
     impl Convert<raw::git_config_level_t> for ConfigLevel {
         fn convert(&self) -> raw::git_config_level_t {
             match *self {
+                ConfigLevel::ProgramData => raw::GIT_CONFIG_LEVEL_PROGRAMDATA,
                 ConfigLevel::System => raw::GIT_CONFIG_LEVEL_SYSTEM,
                 ConfigLevel::XDG => raw::GIT_CONFIG_LEVEL_XDG,
                 ConfigLevel::Global => raw::GIT_CONFIG_LEVEL_GLOBAL,
