@@ -12,7 +12,7 @@ use {Branches, BranchType, Index, Config, Oid, Blob, Branch, Commit, Tree};
 use {AnnotatedCommit, MergeOptions, SubmoduleIgnore, SubmoduleStatus};
 use {ObjectType, Tag, Note, Notes, StatusOptions, Statuses, Status, Revwalk};
 use {RevparseMode, RepositoryInitMode, Reflog, IntoCString, Describe};
-use {DescribeOptions, TreeBuilder, Diff, DiffOptions};
+use {DescribeOptions, TreeBuilder, Diff, DiffOptions, PackBuilder};
 use build::{RepoBuilder, CheckoutBuilder};
 use string_array::StringArray;
 use oid_array::OidArray;
@@ -81,8 +81,9 @@ impl Repository {
     pub fn open_ext<P, O, I>(path: P,
                              flags: RepositoryOpenFlags,
                              ceiling_dirs: I)
-                            -> Result<Repository, Error>
-            where P: AsRef<Path>, O: AsRef<OsStr>, I: IntoIterator<Item=O> {
+                             -> Result<Repository, Error>
+            where P: AsRef<Path>, O: AsRef<OsStr>, I: IntoIterator<Item=O>
+    {
         init();
         let path = try!(path.as_ref().into_c_string());
         let ceiling_dirs_os = try!(env::join_paths(ceiling_dirs));
@@ -276,6 +277,21 @@ impl Repository {
         }
     }
 
+    /// Set the path to the working directory for this repository.
+    ///
+    /// If `update_link` is true, create/update the gitlink file in the workdir
+    /// and set config "core.worktree" (if workdir is not the parent of the .git
+    /// directory).
+    pub fn set_workdir(&self, path: &Path, update_gitlink: bool)
+                       -> Result<(), Error> {
+        let path = try!(path.into_c_string());
+        unsafe {
+            try_call!(raw::git_repository_set_workdir(self.raw(), path,
+                                                      update_gitlink));
+        }
+        Ok(())
+    }
+
     /// Get the currently active namespace for this repository.
     ///
     /// If there is no namespace, or the namespace is not a valid utf8 string,
@@ -447,7 +463,7 @@ impl Repository {
     /// will be left alone, however.)
     ///
     /// The `target` is a commit-ish to which the head should be moved to. The
-    /// object can either be a commit or a tag, but tags must be derefernceable
+    /// object can either be a commit or a tag, but tags must be dereferenceable
     /// to a commit.
     ///
     /// The `checkout` options will only be used for a hard reset.
@@ -675,6 +691,13 @@ impl Repository {
         }
     }
 
+    /// Set the Index file for this repository.
+    pub fn set_index(&self, index: &mut Index) {
+        unsafe {
+            raw::git_repository_set_index(self.raw(), index.raw());
+        }
+    }
+
     /// Get the configuration file for this repository.
     ///
     /// If a configuration file has not been set, the default config set for the
@@ -835,6 +858,57 @@ impl Repository {
         }
     }
 
+    /// Conditionally create new direct reference.
+    ///
+    /// A direct reference (also called an object id reference) refers directly
+    /// to a specific object id (a.k.a. OID or SHA) in the repository.  The id
+    /// permanently refers to the object (although the reference itself can be
+    /// moved).  For example, in libgit2 the direct ref "refs/tags/v0.17.0"
+    /// refers to OID 5b9fac39d8a76b9139667c26a63e6b3f204b3977.
+    ///
+    /// The direct reference will be created in the repository and written to
+    /// the disk.
+    ///
+    /// Valid reference names must follow one of two patterns:
+    ///
+    /// 1. Top-level names must contain only capital letters and underscores,
+    ///    and must begin and end with a letter.  (e.g.  "HEAD", "ORIG_HEAD").
+    /// 2. Names prefixed with "refs/" can be almost anything.  You must avoid
+    ///    the characters `~`, `^`, `:`, `\\`, `?`, `[`, and `*`, and the
+    ///    sequences ".." and "@{" which have special meaning to revparse.
+    ///
+    /// This function will return an error if a reference already exists with
+    /// the given name unless `force` is true, in which case it will be
+    /// overwritten.
+    ///
+    /// The message for the reflog will be ignored if the reference does not
+    /// belong in the standard set (HEAD, branches and remote-tracking
+    /// branches) and it does not have a reflog.
+    ///
+    /// It will return GIT_EMODIFIED if the reference's value at the time of
+    /// updating does not match the one passed through `current_id` (i.e. if the
+    /// ref has changed since the user read it).
+    pub fn reference_matching(&self,
+                              name: &str,
+                              id: Oid,
+                              force: bool,
+                              current_id: Oid,
+                              log_message: &str) -> Result<Reference, Error> {
+        let name = try!(CString::new(name));
+        let log_message = try!(CString::new(log_message));
+        let mut raw = 0 as *mut raw::git_reference;
+        unsafe {
+            try_call!(raw::git_reference_create_matching(&mut raw,
+                                                         self.raw(),
+                                                         name,
+                                                         id.raw(),
+                                                         force,
+                                                         current_id.raw(),
+                                                         log_message));
+            Ok(Binding::from_raw(raw))
+        }
+    }
+
     /// Create a new symbolic reference.
     ///
     /// This function will return an error if a reference already exists with
@@ -856,6 +930,39 @@ impl Repository {
         }
     }
 
+    /// Create a new symbolic reference.
+    ///
+    /// This function will return an error if a reference already exists with
+    /// the given name unless force is true, in which case it will be
+    /// overwritten.
+    ///
+    /// It will return GIT_EMODIFIED if the reference's value at the time of
+    /// updating does not match the one passed through current_value (i.e. if
+    /// the ref has changed since the user read it).
+    pub fn reference_symbolic_matching(&self,
+                                       name: &str,
+                                       target: &str,
+                                       force: bool,
+                                       current_value: &str,
+                                       log_message: &str)
+                                       -> Result<Reference, Error> {
+        let name = try!(CString::new(name));
+        let target = try!(CString::new(target));
+        let current_value = try!(CString::new(current_value));
+        let log_message = try!(CString::new(log_message));
+        let mut raw = 0 as *mut raw::git_reference;
+        unsafe {
+            try_call!(raw::git_reference_symbolic_create_matching(&mut raw,
+                                                                  self.raw(),
+                                                                  name,
+                                                                  target,
+                                                                  force,
+                                                                  current_value,
+                                                                  log_message));
+            Ok(Binding::from_raw(raw))
+        }
+    }
+
     /// Lookup a reference to one of the objects in a repository.
     pub fn find_reference(&self, name: &str) -> Result<Reference, Error> {
         let name = try!(CString::new(name));
@@ -1389,6 +1496,24 @@ impl Repository {
         Ok(())
     }
 
+    /// Check if the given reference has a reflog.
+    pub fn reference_has_log(&self, name: &str) -> Result<bool, Error> {
+        let name = try!(CString::new(name));
+        let ret = unsafe {
+            try_call!(raw::git_reference_has_log(self.raw, name))
+        };
+        Ok(ret != 0)
+    }
+
+    /// Ensure that the given reference has a reflog.
+    pub fn reference_ensure_log(&self, name: &str) -> Result<(), Error> {
+        let name = try!(CString::new(name));
+        unsafe {
+            try_call!(raw::git_reference_ensure_log(self.raw, name));
+        }
+        Ok(())
+    }
+
     /// Describes a commit
     ///
     /// Performs a describe operation on the current commit and the worktree.
@@ -1455,6 +1580,26 @@ impl Repository {
         }
     }
 
+    /// Create a diff between two index objects.
+    ///
+    /// The first index will be used for the "old_file" side of the delta, and
+    /// the second index will be used for the "new_file" side of the delta.
+    pub fn diff_index_to_index(&self,
+                               old_index: &Index,
+                               new_index: &Index,
+                               opts: Option<&mut DiffOptions>)
+                               -> Result<Diff, Error> {
+        let mut ret = 0 as *mut raw::git_diff;
+        unsafe {
+            try_call!(raw::git_diff_index_to_index(&mut ret,
+                                                   self.raw(),
+                                                   old_index.raw(),
+                                                   new_index.raw(),
+                                                   opts.map(|s| s.raw())));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
     /// Create a diff between the repository index and the workdir directory.
     ///
     /// This matches the `git diff` command.  See the note below on
@@ -1531,6 +1676,16 @@ impl Repository {
             Ok(Binding::from_raw(ret))
         }
     }
+
+    /// Create a PackBuilder
+    pub fn packbuilder(&self) -> Result<PackBuilder, Error> {
+        let mut ret = 0 as *mut raw::git_packbuilder;
+        unsafe {
+            try_call!(raw::git_packbuilder_new(&mut ret, self.raw()));
+            Ok(Binding::from_raw(ret))
+        }
+    }
+
 }
 
 impl Binding for Repository {
@@ -1813,8 +1968,9 @@ mod tests {
         let err = Repository::open_ext(&subdir, ::REPOSITORY_OPEN_NO_SEARCH, &[] as &[&OsStr]).err().unwrap();
         assert_eq!(err.code(), ::ErrorCode::NotFound);
 
-        let err = Repository::open_ext(&subdir, ::RepositoryOpenFlags::empty(), &[&subdir]).err().unwrap();
-        assert_eq!(err.code(), ::ErrorCode::NotFound);
+        assert!(Repository::open_ext(&subdir,
+                                     ::RepositoryOpenFlags::empty(),
+                                     &[&subdir]).is_ok());
     }
 
     fn graph_repo_init() -> (TempDir, Repository) {
@@ -1862,6 +2018,20 @@ mod tests {
         assert!(!repo.graph_descendant_of(head_parent_id, head_id).unwrap());
     }
 
+    #[test]
+    fn smoke_reference_has_log_ensure_log() {
+        let (_td, repo) = ::test::repo_init();
+
+        assert_eq!(repo.reference_has_log("HEAD").unwrap(), true);
+        assert_eq!(repo.reference_has_log("refs/heads/master").unwrap(), true);
+        assert_eq!(repo.reference_has_log("NOT_HEAD").unwrap(), false);
+        let master_oid = repo.revparse_single("master").unwrap().id();
+        assert!(repo.reference("NOT_HEAD", master_oid, false, "creating a new branch").is_ok());
+        assert_eq!(repo.reference_has_log("NOT_HEAD").unwrap(), false);
+        assert!(repo.reference_ensure_log("NOT_HEAD").is_ok());
+        assert_eq!(repo.reference_has_log("NOT_HEAD").unwrap(), true);
+    }
+
     #[test]
     fn smoke_set_head() {
         let (_td, repo) = ::test::repo_init();
@@ -1918,6 +2088,8 @@ mod tests {
         let commit2 = repo.find_commit(oid2).unwrap();
         println!("created oid2 {:?}", oid2);
 
+        t!(repo.reset(commit1.as_object(), ResetType::Hard, None));
+
         // create commit oid3 on branchB
         let mut index = repo.index().unwrap();
         let p = Path::new(repo.workdir().unwrap()).join("file_b");
@@ -1966,7 +2138,7 @@ mod tests {
         }
         assert!(found_oid2);
         assert!(found_oid3);
-	assert_eq!(merge_bases.len(), 2);
+	    assert_eq!(merge_bases.len(), 2);
     }
 
     #[test]
