@@ -6,10 +6,10 @@ use std::io;
 use std::marker;
 use std::path::{Path, Component};
 
-use entry::EntryFields;
+use entry::{EntryFields, EntryIo};
 use error::TarError;
 use other;
-use {Entry, Header};
+use {Entry, Header, GnuSparseHeader, GnuExtSparseHeader};
 
 /// A top-level representation of an archive file.
 ///
@@ -20,6 +20,8 @@ pub struct Archive<R: ?Sized + Read> {
 
 pub struct ArchiveInner<R: ?Sized> {
     pos: Cell<u64>,
+    unpack_xattrs: bool,
+    preserve_permissions: bool,
     obj: RefCell<::AlignHigher<R>>,
 }
 
@@ -41,6 +43,8 @@ impl<R: Read> Archive<R> {
     pub fn new(obj: R) -> Archive<R> {
         Archive {
             inner: ArchiveInner {
+                unpack_xattrs: false,
+                preserve_permissions: false,
                 obj: RefCell::new(::AlignHigher(0, obj)),
                 pos: Cell::new(0),
             },
@@ -88,6 +92,26 @@ impl<R: Read> Archive<R> {
         let me: &mut Archive<Read> = self;
         me._unpack(dst.as_ref())
     }
+
+    /// Indicate whether extended file attributes (xattrs on Unix) are preserved
+    /// when unpacking this archive.
+    ///
+    /// This flag is disabled by default and is currently only implemented on
+    /// Unix using xattr support. This may eventually be implemented for
+    /// Windows, however, if other archive implementations are found which do
+    /// this as well.
+    pub fn set_unpack_xattrs(&mut self, unpack_xattrs: bool) {
+        self.inner.unpack_xattrs = unpack_xattrs;
+    }
+
+    /// Indicate whether extended permissions (like suid on Unix) are preserved
+    /// when unpacking this entry.
+    ///
+    /// This flag is disabled by default and is currently only implemented on
+    /// Unix.
+    pub fn set_preserve_permissions(&mut self, preserve: bool) {
+        self.inner.preserve_permissions = preserve;
+    }
 }
 
 impl<'a> Archive<Read + 'a> {
@@ -243,19 +267,22 @@ impl<'a> EntriesFields<'a> {
             return Err(other("archive header checksum mismatch"))
         }
 
-        let size = try!(header.size());
+        let size = try!(header.entry_size());
+
         let ret = EntryFields {
             size: size,
+            data: vec![EntryIo::Data((&self.archive.inner).take(size))],
             header: header,
-            data: (&self.archive.inner).take(size),
             long_pathname: None,
             long_linkname: None,
             pax_extensions: None,
+            unpack_xattrs: self.archive.inner.unpack_xattrs,
+            preserve_permissions: self.archive.inner.preserve_permissions,
         };
 
         // Store where the next entry is, rounding up by 512 bytes (the size of
         // a header);
-        let size = (ret.size + 511) & !(512 - 1);
+        let size = (size + 511) & !(512 - 1);
         self.next += size;
 
         Ok(Some(ret.into_entry()))
@@ -270,7 +297,6 @@ impl<'a> EntriesFields<'a> {
         let mut gnu_longlink = None;
         let mut pax_extensions = None;
         let mut processed = 0;
-
         loop {
             processed += 1;
             let entry = match try!(self.next_entry_raw()) {
@@ -316,9 +342,101 @@ impl<'a> EntriesFields<'a> {
             fields.long_pathname = gnu_longname;
             fields.long_linkname = gnu_longlink;
             fields.pax_extensions = pax_extensions;
+            try!(self.parse_sparse_header(&mut fields));
             return Ok(Some(fields.into_entry()))
         }
     }
+
+    fn parse_sparse_header(&mut self,
+                           entry: &mut EntryFields<'a>) -> io::Result<()> {
+        if !entry.header.entry_type().is_gnu_sparse() {
+            return Ok(())
+        }
+        let gnu = match entry.header.as_gnu() {
+            Some(gnu) => gnu,
+            None => return Err(other("sparse entry type listed but not GNU header")),
+        };
+
+        // Sparse files are represented internally as a list of blocks that are
+        // read. Blocks are either a bunch of 0's or they're data from the
+        // underlying archive.
+        //
+        // Blocks of a sparse file are described by the `GnuSparseHeader`
+        // structure, some of which are contained in `GnuHeader` but some of
+        // which may also be contained after the first header in further
+        // headers.
+        //
+        // We read off all the blocks here and use the `add_block` function to
+        // incrementally add them to the list of I/O block (in `entry.data`).
+        // The `add_block` function also validates that each chunk comes after
+        // the previous, we don't overrun the end of the file, and each block is
+        // aligned to a 512-byte boundary in the archive itself.
+        //
+        // At the end we verify that the sparse file size (`Header::size`) is
+        // the same as the current offset (described by the list of blocks) as
+        // well as the amount of data read equals the size of the entry
+        // (`Header::entry_size`).
+        entry.data.truncate(0);
+
+        let mut cur = 0;
+        let mut remaining = entry.size;
+        {
+            let data = &mut entry.data;
+            let reader = &self.archive.inner;
+            let size = entry.size;
+            let mut add_block = |block: &GnuSparseHeader| -> io::Result<_> {
+                if block.is_empty() {
+                    return Ok(())
+                }
+                let off = try!(block.offset());
+                let len = try!(block.length());
+
+                if (size - remaining) % 512 != 0 {
+                    return Err(other("previous block in sparse file was not \
+                                      aligned to 512-byte boundary"))
+                } else if off < cur {
+                    return Err(other("out of order or overlapping sparse \
+                                      blocks"))
+                } else if cur < off {
+                    let block = io::repeat(0).take(off - cur);
+                    data.push(EntryIo::Pad(block));
+                }
+                cur = try!(off.checked_add(len).ok_or_else(|| {
+                    other("more bytes listed in sparse file than u64 can hold")
+                }));
+                remaining = try!(remaining.checked_sub(len).ok_or_else(|| {
+                    other("sparse file consumed more data than the header \
+                           listed")
+                }));
+                data.push(EntryIo::Data(reader.take(len)));
+                Ok(())
+            };
+            for block in gnu.sparse.iter() {
+                try!(add_block(block))
+            }
+            if gnu.is_extended() {
+                let mut ext = GnuExtSparseHeader::new();
+                ext.isextended[0] = 1;
+                while ext.is_extended() {
+                    try!(read_all(&mut &self.archive.inner, ext.as_mut_bytes()));
+                    self.next += 512;
+                    for block in ext.sparse.iter() {
+                        try!(add_block(block));
+                    }
+                }
+            }
+        }
+        if cur != try!(gnu.real_size()) {
+            return Err(other("mismatch in sparse file chunks and \
+                              size in header"))
+        }
+        entry.size = cur;
+        if remaining > 0 {
+            return Err(other("mismatch in sparse file chunks and \
+                              entry size in header"))
+        }
+        Ok(())
+    }
 }
 
 impl<'a> Iterator for EntriesFields<'a> {
