@@ -2,7 +2,7 @@ use std::borrow::Cow;
 use std::cmp;
 use std::fs;
 use std::io::prelude::*;
-use std::io;
+use std::io::{self, SeekFrom};
 use std::marker;
 use std::path::Path;
 
@@ -33,7 +33,14 @@ pub struct EntryFields<'a> {
     pub pax_extensions: Option<Vec<u8>>,
     pub header: Header,
     pub size: u64,
-    pub data: io::Take<&'a ArchiveInner<Read + 'a>>,
+    pub data: Vec<EntryIo<'a>>,
+    pub unpack_xattrs: bool,
+    pub preserve_permissions: bool,
+}
+
+pub enum EntryIo<'a> {
+    Pad(io::Take<io::Repeat>),
+    Data(io::Take<&'a ArchiveInner<Read + 'a>>),
 }
 
 impl<'a, R: Read> Entry<'a, R> {
@@ -144,6 +151,26 @@ impl<'a, R: Read> Entry<'a, R> {
     pub fn unpack<P: AsRef<Path>>(&mut self, dst: P) -> io::Result<()> {
         self.fields.unpack(dst.as_ref())
     }
+
+    /// Indicate whether extended file attributes (xattrs on Unix) are preserved
+    /// when unpacking this entry.
+    ///
+    /// This flag is disabled by default and is currently only implemented on
+    /// Unix using xattr support. This may eventually be implemented for
+    /// Windows, however, if other archive implementations are found which do
+    /// this as well.
+    pub fn set_unpack_xattrs(&mut self, unpack_xattrs: bool) {
+        self.fields.unpack_xattrs = unpack_xattrs;
+    }
+
+    /// Indicate whether extended permissions (like suid on Unix) are preserved
+    /// when unpacking this entry.
+    ///
+    /// This flag is disabled by default and is currently only implemented on
+    /// Unix.
+    pub fn set_preserve_permissions(&mut self, preserve: bool) {
+        self.fields.preserve_permissions = preserve;
+    }
 }
 
 impl<'a, R: Read> Read for Entry<'a, R> {
@@ -267,8 +294,21 @@ impl<'a> EntryFields<'a> {
         // as we would normally.
 
         try!(fs::File::create(dst).and_then(|mut f| {
-            if try!(io::copy(self, &mut f)) != self.size {
-                return Err(other("failed to write entire file"));
+            for io in self.data.drain(..) {
+                match io {
+                    EntryIo::Data(mut d) => {
+                        let expected = d.limit();
+                        if try!(io::copy(&mut d, &mut f)) != expected {
+                            return Err(other("failed to write entire file"));
+                        }
+                    }
+                    EntryIo::Pad(d) => {
+                        // TODO: checked cast to i64
+                        let to = SeekFrom::Current(d.limit() as i64);
+                        let size = try!(f.seek(to));
+                        try!(f.set_len(size));
+                    }
+                }
             }
             Ok(())
         }).map_err(|e| {
@@ -286,33 +326,100 @@ impl<'a> EntryFields<'a> {
             }));
         }
         if let Ok(mode) = self.header.mode() {
-            try!(set_perms(dst, mode).map_err(|e| {
+            try!(set_perms(dst, mode, self.preserve_permissions).map_err(|e| {
                 TarError::new(&format!("failed to set permissions to {:o} \
                                         for `{}`", mode, dst.display()), e)
             }));
         }
+        if self.unpack_xattrs {
+            try!(set_xattrs(self, dst));
+        }
         return Ok(());
 
         #[cfg(unix)]
         #[allow(deprecated)] // raw deprecated in 1.8
-        fn set_perms(dst: &Path, mode: u32) -> io::Result<()> {
+        fn set_perms(dst: &Path, mode: u32, preserve: bool) -> io::Result<()> {
             use std::os::unix::raw;
             use std::os::unix::prelude::*;
 
+            let mode = if preserve {
+                mode
+            } else {
+                mode & 0o777
+            };
+
             let perm = fs::Permissions::from_mode(mode as raw::mode_t);
             fs::set_permissions(dst, perm)
         }
         #[cfg(windows)]
-        fn set_perms(dst: &Path, mode: u32) -> io::Result<()> {
+        fn set_perms(dst: &Path, mode: u32, _preserve: bool) -> io::Result<()> {
             let mut perm = try!(fs::metadata(dst)).permissions();
             perm.set_readonly(mode & 0o200 != 0o200);
             fs::set_permissions(dst, perm)
         }
+
+        #[cfg(all(unix, feature = "xattr"))]
+        fn set_xattrs(me: &mut EntryFields, dst: &Path) -> io::Result<()> {
+            use std::os::unix::prelude::*;
+            use std::ffi::OsStr;
+            use xattr;
+
+            let exts = match me.pax_extensions() {
+                Ok(Some(e)) => e,
+                _ => return Ok(()),
+            };
+            let exts = exts.filter_map(|e| e.ok()).filter_map(|e| {
+                let key = e.key_bytes();
+                let prefix = b"SCHILY.xattr.";
+                if key.starts_with(prefix) {
+                    Some((&key[prefix.len()..], e))
+                } else {
+                    None
+                }
+            }).map(|(key, e)| {
+                (OsStr::from_bytes(key), e.value_bytes())
+            });
+
+            for (key, value) in exts {
+                try!(xattr::set(dst, key, value).map_err(|e| {
+                    TarError::new(&format!("failed to set extended \
+                                            attributes to {}. \
+                                            Xattrs: key={:?}, value={:?}.",
+                                           dst.display(),
+                                           key,
+                                           String::from_utf8_lossy(value)),
+                                  e)
+                }));
+            }
+
+            Ok(())
+        }
+        // Windows does not completely support posix xattrs
+        // https://en.wikipedia.org/wiki/Extended_file_attributes#Windows_NT
+        #[cfg(any(windows, not(feature = "xattr")))]
+        fn set_xattrs(_: &mut EntryFields, _: &Path) -> io::Result<()> {
+            Ok(())
+        }
     }
 }
 
 impl<'a> Read for EntryFields<'a> {
     fn read(&mut self, into: &mut [u8]) -> io::Result<usize> {
-        self.data.read(into)
+        loop {
+            match self.data.get_mut(0).map(|io| io.read(into)) {
+                Some(Ok(0)) => { self.data.remove(0); }
+                Some(r) => return r,
+                None => return Ok(0),
+            }
+        }
+    }
+}
+
+impl<'a> Read for EntryIo<'a> {
+    fn read(&mut self, into: &mut [u8]) -> io::Result<usize> {
+        match *self {
+            EntryIo::Pad(ref mut io) => io.read(into),
+            EntryIo::Data(ref mut io) => io.read(into),
+        }
     }
 }
